@@ -1,6 +1,6 @@
 /***************************************************************************
  *
- * Copyright (C) 2018-2020 - ZmartZone Holding BV - www.zmartzone.eu
+ * Copyright (C) 2018-2021 - ZmartZone Holding BV - www.zmartzone.eu
  *
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU Affero General Public License as
@@ -309,8 +309,10 @@ static bool _oauth2_dpop_jti_validate(oauth2_log_t *log,
 
 	oauth2_cache_get(log, verify->cache, clm_jti, &s_value);
 	if (s_value != NULL) {
-		oauth2_error(log, "a token with the same JTI \"%s\" exists in "
-				  "the cache: possible replay attack");
+		oauth2_error(log,
+			     "a token with the same JTI \"%s\" exists in "
+			     "the cache: possible replay attack",
+			     clm_jti);
 		goto end;
 	}
 
